/**
 * Created by jmoutard on 19/07/16.
 */

'use strict';
/*
 * Local Storage Service give data from local storage (HTML5) if not logged in and is capable of giving errors.
 * Errors are:
 *          - 10; No local storage (no HTML5 local storage),
 *          - 11; Error when adding dashboard (Here we just add a newly create dashboard),
 *          - 12; Dashboard not found (local storage stores just one dashboard),
 *          - 13; Can't remove the dashboard.
 *
 * For convenience error code from 10 to 19 belongs to local storage errors.
 * NB: If you want to create a new local dashboard and you already have one, you need to remove the one existing first before create a new one.
 */
angular.module('adf.electre')
    .service('localStorageService', [
        "$q", "storageErrorProvider",
        function ($q, storageErrorProvider) {
            /**
             * @description Return all stored dashboard from the server.
             * @returns {Promise}
             */
            function getAll () {
                var deferred = $q.defer();

                setTimeout(function () {
                    if (typeof(Storage) !== "undefined") {
                        var electre_storage = localStorage.getItem("electre");
                        try {
                            electre_storage = JSON.parse(electre_storage);
                            /* Get the ids and titles of all dashboard  */
                            var response = electre_storage.dashboards.map(function (dashboard) { return {"id": dashboard.id, "title": dashboard.title, state: "local"}; });
                            deferred.resolve(response);
                        } catch (e) {
                            deferred.reject(new storageErrorProvider(12, "No dashboard found."));
                        }
                    } else
                        deferred.reject(new storageErrorProvider(10, "HTML5 local storage not supported."));
                }, 100);

                return deferred.promise;
            }

            /**
             * @description Return a specific stored dashboard from the server.
             * @param {string} id
             * @returns {Promise}
             */
            function get (id) {
                var deferred = $q.defer();

                setTimeout(function () {
                    if (typeof(Storage) !== "undefined") {
                        var electre_storage = localStorage.getItem("electre");
                        try {
                            electre_storage = JSON.parse(electre_storage);
                            var dashboards = electre_storage.dashboards;
                            for (var d=0; d<dashboards.length ; d++) {
                                if (dashboards[d].id === id) {
                                    deferred.resolve(electre_storage.dashboards[d]);
                                    break;
                                }
                            }
                        } catch (e) {
                            deferred.reject(new storageErrorProvider(12, "No dashboard found."));
                        }
                    } else
                        deferred.reject(new storageErrorProvider(10, "HTML5 local storage not supported."));
                }, 100);

                return deferred.promise;
            }

            /**
             * @description Add a dashboard in the local storage.
             * @param {string} id
             * @param {object} data
             * @returns {Promise}
             */
            function add (id, data) {
                var deferred = $q.defer();

                setTimeout(function () {
                    if (typeof(Storage) !== "undefined") {
                        var electre_storage = localStorage.getItem("electre");
                        if (!electre_storage) {
                            try {
                                electre_storage = {"dashboards": [data]};
                                localStorage.setItem("electre", JSON.stringify(electre_storage));
                                deferred.resolve(true);
                            } catch (e) {
                                deferred.reject(new storageErrorProvider(11, "Can't add dashboard."));
                            }
                        } else {
                            try {
                                electre_storage = JSON.parse(electre_storage);
                                var updated = false;
                                for (var d=0 ; d<electre_storage.dashboards.length ; d++) {
                                    if (electre_storage.dashboards[d].id === id) {
                                        electre_storage.dashboards[d] = data;
                                        updated = true;
                                        break;
                                    }
                                }
                                if (!updated) electre_storage.dashboards.push(data);
                                localStorage.setItem("electre", JSON.stringify(electre_storage));
                                deferred.resolve(true);
                            } catch (e) {
                                deferred.reject(new storageErrorProvider(11, "Can't add dashboard."));
                            }
                        }
                    } else
                        deferred.reject(new storageErrorProvider(10, "HTML5 local storage not supported."));
                }, 100);

                return deferred.promise;
            }

            /**
             * @description Remove a dashboard onto the server.
             * @param {string} id
             * @returns {Promise}
             */
            function remove (id) {
                var deferred = $q.defer();

                setTimeout(function () {
                    if (typeof(Storage) !== "undefined") {
                        var electre_storage = localStorage.getItem("electre");
                        try {
                            electre_storage = JSON.parse(electre_storage);
                            /* Filter the dashboard with different id as the one passed by argument */
                            electre_storage.dashboards = electre_storage.dashboards.filter(function (dashboard) { return dashboard.id !== id; });
                            localStorage.setItem("electre", JSON.stringify(electre_storage));
                            deferred.resolve(true);
                        } catch (e) {
                            deferred.reject(new storageErrorProvider(13, "Can't remove dashboard."));
                        }
                    } else
                        deferred.reject(new storageErrorProvider(10, "HTML5 local storage not supported."));
                }, 100);

                return deferred.promise;
            }

            return { getAll: getAll, get: get, add: add, remove: remove};
        }
    ]);